/* singhead.c */

#define PROGRAM_NAME    "singhead"
#define PROGRAM_VER     "1.00"
#define PROGRAM_DATE    "07/22/92"
/*
 * Purpose: Removes all but a single set of initials from the head of each
 *      paragraph in a packet mail reply. Can optionally add a signature.
 */
#define PROGRAM_SYNTAX \
"\nsinghead <reply_file> \"<quote_character>\" [\"signature\"|@signature_file]\n"
/*
 * Returns: DOS errorlevel 0 if successful, >0 if fail
 */


#ifndef TRUE
#define FALSE   0
#define TRUE    1
#endif

#define MAXLINE 1024
#define QUIT(E) free(s); exit(E);
#define BOX     (char)254

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

struct  singh
{
	FILE    *inf;
	FILE    *ouf;
	char    ins[MAXLINE];
	char    tempname[_MAX_PATH];
	char    tempdrive[_MAX_DRIVE];
	char    tempdir[_MAX_DIR];
};
typedef struct singh    SINGHEAD;

int appendsig(char *signature, char *ins, FILE *ouf);
FILE *openfile(char *fname, char *fmode);
void striplines(FILE *inf, FILE *ouf, char *ins, char quotechar);

int appendsig(char *signature, char *ins, FILE *ouf)
{
    char        *a;
    FILE        *sigfile;

    if(*signature=='@') /* if signature file specified */
    {
	if((sigfile=openfile(signature+1, "r"))==NULL)
	    return 1;
	fputc('\n', ouf);
	while((fgets(ins, MAXLINE, sigfile))!=NULL)
	    fputs(ins, ouf);
    }
    else                /* else signature is command line param */
    {
	/*
	   I'm using 75 as the maximum line length. Take half that (37),
	   and subtract half the length of the signature from it. This
	   gives us the starting position, within the line, for a
	   centered signature.
	*/
	a=ins+37-(strlen(signature)>>1);
	if(a<ins)       /* check for buffer overrun */
	{
	    fputs("Signature too long\n", stderr);
	    return 1;
	}
	strcpy(a, signature);   /* drop the signature in at that position */
	strcat(a, "\n");        /* add a newline */
	for(a--;a>=ins;a--)   /* and backfill with spaces */
	    *a=(char)32;
	fputc('\n', ouf);
	fputs(ins, ouf);
    }
    return 0;
}

void striplines(FILE *inf, FILE *ouf, char *ins, char quotechar)
{
    int         newpara=TRUE;
    char        *q;

    fputs("Processing the text\n", stderr);
    /* read file in a line at a time */
    while((fgets(ins, MAXLINE, inf))!=NULL)
    {
	if(*ins=='\n')                  /* blank line means new paragraph */
	    newpara=TRUE;
	else
	    if((q=strchr(ins,(int)quotechar))!=NULL) /* quotechar found? */
		if(newpara)             /* ignore if new paragraph */
		    newpara=FALSE;
		else
		    for(q--;q>=ins;q--) /* else overwrite initials */
			*q=(char)32;
	fputs(ins, ouf);                /* print line to outfile */
    }
}

FILE *openfile(char *fname, char *fmode)
{
    FILE        *fp;
    char        err[_MAX_PATH+13];

    /*
       Prints an error message if file open fails.
       Otherwise, works just like fopen().
    */

    sprintf(err, "**%s** %s", PROGRAM_NAME, fname);
    if((fp=fopen(fname, fmode))==NULL)
	perror(err);
    return fp;
}

void main(int argc, char **argv)
{
    SINGHEAD    *s;     /* structure pointer for variables */

    if(argc<3 || argc>4)
    {
	fputs(PROGRAM_SYNTAX, stderr);
	exit(1);
    }

    fprintf(stderr, "%s %c v%s %c %s\n", PROGRAM_NAME, BOX, PROGRAM_VER,
     BOX, PROGRAM_DATE);
    if((s=(SINGHEAD *)malloc(sizeof (SINGHEAD)))==NULL)
    {
	fprintf(stderr,"**%s** Cannot allocate memory for buffer\n",
	 PROGRAM_NAME);
	exit(2);
    }

    /*
       Attempt to open input file.
       If input file not found and no signature requested, quit.
       If input file not found and signature is requested, open in w+ mode.
    */
    s->inf=openfile(argv[1], "r");
    if(s->inf==NULL)
    {
	if(argc==3)
	{
	    QUIT(3);
	}
	else if((s->inf=openfile(argv[1], "w+"))==NULL)
	{
	    QUIT(4);
	}
	else
	    fprintf(stderr, "%s created\n", argv[1]);
    }
    else
	fprintf(stderr, "Opened input file %s\n", argv[1]);

    /* prepare pathname for temporary file */
    _fullpath(s->tempname, argv[1], _MAX_PATH);
    _splitpath(s->tempname, s->tempdrive, s->tempdir, NULL, NULL);
    _makepath(s->tempname, s->tempdrive, s->tempdir, "singhead", ".tmp");

    /* open temporary file for output */
    if((s->ouf=openfile(s->tempname, "w"))==NULL)
    {
	QUIT(5);
    }
    else
	fputs("Output file created\n", stderr);

    /* process the text */
    striplines(s->inf, s->ouf, s->ins, *argv[2]);

    /* append signature, if requested */
    if(argc=4)
	if(appendsig(argv[3], s->ins, s->ouf))
	    fprintf(stderr, "**%s** Signature not appended\n", PROGRAM_NAME);
	else
	    fputs("Signature appended\n", stderr);

    fcloseall();
    remove(argv[1]);                    /* delete the input file */
    rename(s->tempname, argv[1]);       /* rename tempfile as original file */
    fprintf(stderr,"Input file deleted, and output renamed %s\n", argv[1]);
    QUIT(0);
}
